@@ -90,8 +90,13 @@ static int session_dispatch_leader_pidfd(sd_event_source *es, int fd, uint32_t r
         Session *s = ASSERT_PTR(userdata);
 
         assert(s->leader.fd == fd);
+
+        s->leader_pidfd_event_source = sd_event_source_unref(s->leader_pidfd_event_source);
+
         session_stop(s, /* force= */ false);
 
+        session_add_to_gc_queue(s);
+
         return 1;
 }
 
@@ -947,7 +952,6 @@ int session_stop(Session *s, bool force) {
                 return 0;
 
         s->timer_event_source = sd_event_source_unref(s->timer_event_source);
-        s->leader_pidfd_event_source = sd_event_source_unref(s->leader_pidfd_event_source);
 
         if (s->seat)
                 seat_evict_position(s->seat, s);
@@ -1255,6 +1259,8 @@ static int session_dispatch_fifo(sd_event_source *es, int fd, uint32_t revents,
         session_remove_fifo(s);
         session_stop(s, /* force = */ false);
 
+        session_add_to_gc_queue(s);
+
         return 1;
 }
 
@@ -1384,10 +1390,20 @@ SessionState session_get_state(Session *s) {
 int session_kill(Session *s, KillWho who, int signo) {
         assert(s);
 
-        if (!s->scope)
-                return -ESRCH;
+        switch (who) {
 
-        return manager_kill_unit(s->manager, s->scope, who, signo, NULL);
+        case KILL_ALL:
+                if (!s->scope)
+                        return -ESRCH;
+
+                return manager_kill_unit(s->manager, s->scope, KILL_ALL, signo, NULL);
+
+        case KILL_LEADER:
+                return pidref_kill(&s->leader, signo);
+
+        default:
+                assert_not_reached();
+        }
 }
 
 static int session_open_vt(Session *s, bool reopen) {
