@@ -53,6 +53,7 @@ int machine_new(MachineClass class, const char *name, Machine **ret) {
                 return -ENOMEM;
 
         *m = (Machine) {
+                .class = class,
                 .leader = PIDREF_NULL,
                 .vsock_cid = VMADDR_CID_ANY,
         };
@@ -63,8 +64,6 @@ int machine_new(MachineClass class, const char *name, Machine **ret) {
                         return -ENOMEM;
         }
 
-        m->class = class;
-
         *ret = TAKE_PTR(m);
         return 0;
 }
@@ -572,6 +571,9 @@ bool machine_may_gc(Machine *m, bool drop_not_started) {
         if (m->class == MACHINE_HOST)
                 return false;
 
+        if (!pidref_is_set(&m->leader))
+                return true;
+
         if (drop_not_started && !m->started)
                 return true;
 
