@@ -956,7 +956,7 @@ static int systemctl_parse_argv(int argc, char *argv[]) {
 
                 case ARG_WHAT:
                         if (isempty(optarg))
-                                return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "--what= requires arguments.");
+                                return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "--what= requires arguments (see --what=help).");
 
                         for (const char *p = optarg;;) {
                                 _cleanup_free_ char *k = NULL;
@@ -973,7 +973,8 @@ static int systemctl_parse_argv(int argc, char *argv[]) {
                                              "cache\n"
                                              "logs\n"
                                              "configuration\n"
-                                             "fdstore");
+                                             "fdstore\n"
+                                             "all");
                                         return 0;
                                 }
 
@@ -1073,6 +1074,10 @@ static int systemctl_parse_argv(int argc, char *argv[]) {
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
                                        "Cannot access user instance remotely.");
 
+        if (arg_transport == BUS_TRANSPORT_CAPSULE && arg_runtime_scope != RUNTIME_SCOPE_USER)
+                return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
+                                       "Cannot access system instance with --capsule=/-C.");
+
         if (arg_wait && arg_no_block)
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
                                        "--wait may not be combined with --no-block.");
