@@ -763,12 +763,13 @@ static int enumerate_sysv(const LookupPaths *lp, Hashmap *all_services) {
                                 return log_oom();
 
                         log_struct(LOG_WARNING,
-                                   LOG_MESSAGE("SysV service '%s' lacks a native systemd unit file. "
-                                               "%s Automatically generating a unit file for compatibility. Please update package to include a native systemd unit file, in order to make it safe, robust and future-proof. "
+                                   LOG_MESSAGE("SysV service '%s' lacks a native systemd unit file, "
+                                               "automatically generating a unit file for compatibility.\n"
+                                               "Please update package to include a native systemd unit file.\n"
                                                "%s This compatibility logic is deprecated, expect removal soon. %s",
                                                fpath,
-                                               special_glyph(SPECIAL_GLYPH_RECYCLING),
-                                               special_glyph(SPECIAL_GLYPH_WARNING_SIGN), special_glyph(SPECIAL_GLYPH_WARNING_SIGN)),
+                                               special_glyph(SPECIAL_GLYPH_WARNING_SIGN),
+                                               special_glyph(SPECIAL_GLYPH_WARNING_SIGN)),
                                    "MESSAGE_ID=" SD_MESSAGE_SYSV_GENERATOR_DEPRECATED_STR,
                                    "SYSVSCRIPT=%s", fpath,
                                    "UNIT=%s", name);
