@@ -1135,6 +1135,18 @@ TEST(ASSERT) {
         ASSERT_SIGNAL(ASSERT_OK_ERRNO(-1), SIGABRT);
         ASSERT_SIGNAL(ASSERT_OK_ERRNO(-ENOANO), SIGABRT);
 
+        ASSERT_OK_ZERO_ERRNO(0);
+        ASSERT_SIGNAL(ASSERT_OK_ZERO_ERRNO(1), SIGABRT);
+        ASSERT_SIGNAL(ASSERT_OK_ZERO_ERRNO(255), SIGABRT);
+        ASSERT_SIGNAL(ASSERT_OK_ZERO_ERRNO(-1), SIGABRT);
+        ASSERT_SIGNAL(ASSERT_OK_ZERO_ERRNO(-ENOANO), SIGABRT);
+
+        ASSERT_OK_EQ_ERRNO(0, 0);
+        ASSERT_SIGNAL(ASSERT_OK_EQ_ERRNO(1, 0), SIGABRT);
+        ASSERT_SIGNAL(ASSERT_OK_EQ_ERRNO(255, 5), SIGABRT);
+        ASSERT_SIGNAL(ASSERT_OK_EQ_ERRNO(-1, 0), SIGABRT);
+        ASSERT_SIGNAL(ASSERT_OK_EQ_ERRNO(-ENOANO, 0), SIGABRT);
+
         ASSERT_FAIL(-ENOENT);
         ASSERT_FAIL(-EPERM);
         ASSERT_SIGNAL(ASSERT_FAIL(0), SIGABRT);
